@@ -8,7 +8,7 @@ require Exporter;
 @ISA = qw(Exporter);
 
 @EXPORT_OK = qw(
- crc8 crcccitt crc16 crcopenpgparmor crc32 crc64 crc
+ crc8 crcsaej1850 crcccitt crc16 crcopenpgparmor crc32 crc64 crc
  crc_hex crc_base64
  crcccitt_hex crcccitt_base64
  crc8_hex crc8_base64
@@ -18,9 +18,9 @@ require Exporter;
  crc64_hex crc64_base64
 );
 
-$VERSION    = '0.18';
+$VERSION    = '0.24';
 $XS_VERSION = $VERSION;
-$VERSION    = eval $VERSION;
+#$VERSION    = eval $VERSION;
 
 eval {
   # PERL_DL_NONLAZY must be false, or any errors in loading will just
@@ -69,13 +69,13 @@ sub _tabinit($$$) {
     $r = $i if $ref;
     for (my $j=0; $j<8; $j++) {
       if ($ref) {
-	$r = ($r>>1)^($r&1&&$poly)
+        $r = ($r>>1)^($r&1&&$poly)
       } else {
-	if ($r&(1<<($width-1))) {
-	  $r = ($r<<1)^$poly
-	} else {
-	  $r = ($r<<1)
-	}
+        if ($r&(1<<($width-1))) {
+          $r = ($r<<1)^$poly
+        } else {
+          $r = ($r<<1)
+        }
       }
     }
     my $x=$r&2**$width-1;
@@ -125,6 +125,7 @@ ENOXS
 %_typedef = (
 # name,  [width,init,xorout,refout,poly,refin,cont);
   crc8 => [8,0,0,0,0x07,0,0],
+  crcsaej1850 => [8,0xff,0xff,0,0x1D,0,0],
   crcccitt => [16,0xffff,0,0,0x1021,0,0],
   crc16 => [16,0,0,1,0x8005,1,0],
   crcopenpgparmor => [24,0xB704CE,0,0,0x864CFB,0,0],
@@ -134,6 +135,7 @@ ENOXS
 sub new {
   my $that=shift;
   my %params=@_;
+  die if defined($params{type}) && !exists($_typedef{$params{type}}) && $params{type} ne 'crc64';
   my $class = ref($that) || $that;
   my $self = {map { ($_ => $params{$_}) }
                       qw(type width init xorout refout poly refin cont)};
@@ -161,14 +163,14 @@ sub reset {
     $self->{refin} = $typeparams->[5],
     $self->{cont} = $typeparams->[6],
   }
-  $self->{_tab} = _tabinit($self->{width}, $self->{poly}, $self->{refin});
+  $self->{_tab} = defined($self->{width})?_tabinit($self->{width}, $self->{poly}, $self->{refin}):undef;
   $self->{_data} = undef;
   $self
 }
 
 #########################################
 # Private output converter functions:
-sub _encode_hex { sprintf "%x", $_[0] }
+sub _encode_hex { sprintf "%0${_[1]}x", $_[0] }
 
 sub _encode_base64 {
   my ($res, $padding, $in) = ("", undef, $_[0]);
@@ -223,8 +225,13 @@ sub digest {
   my $crc;
   if (!$self->{_crc}) {
     my $init = $self->{init};
-    $crc =_crc($self->{_data},$self->{width},$init,$self->{xorout},
+    if (defined($self->{type}) && $self->{type} eq 'crc64' ||
+        defined($self->{width}) && $self->{width} eq 64) {
+      $crc = _crc64($self->{_data});
+    } else {
+      $crc =_crc($self->{_data},$self->{width},$init,$self->{xorout},
 	 $self->{refin},$self->{refout},$self->{cont},$self->{_tab});
+    }
   } else {
     $crc = $self->{_crc};
     $self->{_crc} = undef;
@@ -234,7 +241,7 @@ sub digest {
 }
 
 sub hexdigest {
-  _encode_hex($_[0]->digest)
+  _encode_hex($_[0]->digest, $_[0]->{width}/4)
 }
 
 sub b64digest {
@@ -251,7 +258,10 @@ sub clone {
     poly => $self->{poly},
     refin => $self->{refin},
     refout => $self->{refout},
-    _data => $self->{_data}
+    _data => $self->{_data},
+    cont => $self->{cont},
+    _tab => $self->{_tab},
+    _crc => $self->{_crc}
   };
   bless $clone, ref $self || $self;
 }
@@ -278,6 +288,11 @@ sub _cont {
 
 sub crc8 { _cont($_[0],$_[1],@{$_typedef{crc8}}) }
 
+# CRC-SAE-J1850 standard
+# poly: 1D, width: 8, init: ff, refin: no, refout: no, xorout: FF
+
+sub crcsaej1850 { _cont($_[0],$_[1],@{$_typedef{crcsaej1850}}) }
+
 # CRC-CCITT standard
 # poly: 1021, width: 16, init: ffff, refin: no, refout: no, xorout: no
 
@@ -304,33 +319,33 @@ sub crc32 { _cont($_[0],$_[1],@{$_typedef{crc32}}) }
 # CRC64
 # special XS implementation (_crc64)
 
-sub crc64 { _crc64($_[0],$_[1]) }
+sub crc64 { _crc64($_[0],defined($_[1])?$_[1]:0) }
 
-sub crc_hex { _encode_hex &crc }
+sub crc_hex { _encode_hex(&crc,2) }
 
 sub crc_base64 { _encode_base64 &crc }
 
-sub crc8_hex { _encode_hex &crc8 }
+sub crc8_hex { _encode_hex(&crc8,2) }
 
 sub crc8_base64 { _encode_base64 &crc8 }
 
-sub crcccitt_hex { _encode_hex &crcccitt }
+sub crcccitt_hex { _encode_hex(&crcccitt,4) }
 
 sub crcccitt_base64 { _encode_base64 &crcccitt }
 
-sub crc16_hex { _encode_hex &crc16 }
+sub crc16_hex { _encode_hex(&crc16,4) }
 
 sub crc16_base64 { _encode_base64 &crc16 }
 
-sub crcopenpgparmor_hex { _encode_hex &crcopenpgparmor }
+sub crcopenpgparmor_hex { _encode_hex(&crcopenpgparmor,6) }
 
 sub crcopenpgparmor_base64 { _encode_base64 &crcopenpgparmor }
 
-sub crc32_hex { _encode_hex &crc32 }
+sub crc32_hex { _encode_hex(&crc32,8) }
 
 sub crc32_base64 { _encode_base64 &crc32 }
 
-sub crc64_hex { _encode_hex &crc64 }
+sub crc64_hex { _encode_hex(&crc64,16) }
 
 sub crc64_base64 { _encode_base64 &crc64 }
 
